/*
 * Licensed Materials - Property of IBM
 * 5648-C63
 * (C) Copyright IBM Corp. 1999, 2001
 */

#ifndef PERLNODE
#define PERLNODE 1
#ifdef __cplusplus
extern "C" {
#endif

#include <BipCni.h>
#include "PluginUtil.h"
#include "Perl.h"
#include "wmqiperl.h"


#ifndef max
#define max(x, y) (((x) > (y)) ? (x) : (y))
#endif

#ifndef min
#define min(x,y) (((x)<(y)) ? (x) : (y))
#endif

/* ---------------------------------------------------------------- */
/*                     Internal constants                           */
/*                                                                  */
/* The interface expects some CciChar strings to be passed. These   */
/* have to be constructed from char strings using the supplied      */
/* utilities. The char constants are defined here along with the    */
/* static CciChar variables which will contain these constants      */
/* converted into CciChar format.                                   */
/*                                                                  */
/* ---------------------------------------------------------------- */
#define ELT_NAME_LEN                         256




#define MAX_ATTR_NAME_LEN     128
#define MAX_NODE_NAME_LEN     128
#define MAX_STRING_SIZE       256
#define MAX_PERL_SIZE         0xFFF0
#define MAX_TERMINAL_NAME_LEN 256
#define CNI_TYPE_INTEGER        1
#define CNI_TYPE_STRING         2

#define CCI_CHECK_RC()                                                                        \
   if (rc != CCI_SUCCESS) {                                                                   \
     if (rc == CCI_EXCEPTION) cciRethrowLastException(0);                                     \
     else {                                                                                   \
       return;                                                                                \
     }                                                                                        \
   }

typedef struct attributeTblEntry {
  void*    next;                    /* Pointer to next entry */
  void*    valptr;                  /* Pointer to value */
  int      type;                    /* Value type */
  int      size;                    /* attribute size */
  CciChar  name[MAX_ATTR_NAME_LEN]; /* Attribute name */
} ATTRIBUTE_TBL_ENTRY;

typedef struct terminalListEntry {
  void*        next;                        /* Pointer to next entry */
  CciTerminal* handle;                      /* Handle to terminal object */
  CciChar      name[MAX_TERMINAL_NAME_LEN]; /* Terminal name */
} TERMINAL_LIST_ENTRY;

typedef struct nodeContext {
  CciNode*             nodeObject;                  /* Handle for node object */
  ATTRIBUTE_TBL_ENTRY* attrListHead;                /* Head of linked list of attributes */
  ATTRIBUTE_TBL_ENTRY* attrListPrevious;            /* Previous entry in attribute list */
  TERMINAL_LIST_ENTRY* inputTerminalListHead;       /* Head of linked list of input terminal handles */
  TERMINAL_LIST_ENTRY* inputTerminalListPrevious;   /* Previous entry in input terminal list */
  TERMINAL_LIST_ENTRY* outputTerminalListHead;      /* Head of linked list of output terminal handles */
  TERMINAL_LIST_ENTRY* outputTerminalListPrevious;  /* Previous entry in output terminal list */
  int                  trace;                       /* Flag: Trace active? */
  FILE*                tracefile;                   /* Trace file stream */
  CciChar              nodeName[MAX_NODE_NAME_LEN]; /* Node name */
} NODE_CONTEXT_ST;

/* Function prototypes */

ATTRIBUTE_TBL_ENTRY *insAttrTblEntry(
  NODE_CONTEXT_ST* context,
  CciChar*         attrName,
  int              type
);

void *getInputTerminalHandle(
  NODE_CONTEXT_ST* context,
  CciChar*         terminalName
);

TERMINAL_LIST_ENTRY *insInputTerminalListEntry(
  NODE_CONTEXT_ST* context,
  CciChar*         terminalName
);

void *getOutputTerminalHandle(
  NODE_CONTEXT_ST* context,
  CciChar*         terminalName
);

TERMINAL_LIST_ENTRY *insOutputTerminalListEntry(
  NODE_CONTEXT_ST* context,
  CciChar*         terminalName
);


CciContext* _Perl_createNodeContext(
  CciFactory* factoryObject,
  CciChar*    nodeName,
  CciNode*    nodeObject
);

int _getAttributeName(
  CciContext* context,
  int         index,
  CciChar*    buffer,
  int         bufsize
);

int _setAttribute(
  CciContext* context,
  CciChar*    attrName,
  CciChar*    attrValue
);

int _getAttribute(
  CciContext* context,
  CciChar*    attrName,
  CciChar*    buffer,
  int         bufsize
);

void _Switch_evaluate(
  CciContext* context,
  CciContext* destinationList,
  CciContext* exceptionList,
  CciContext* message
);

void _Perl_evaluate(
  CciContext* context,
  CciContext* destinationList,
  CciContext* exceptionList,
  CciContext* message
);


#ifdef __cplusplus
}

#endif

#endif